package eu.mhsl.minecraft.pixelblocks.pixelblock;

import eu.mhsl.minecraft.pixelblocks.Main;
import org.bukkit.Location;
import org.bukkit.NamespacedKey;
import org.bukkit.block.data.BlockData;
import org.bukkit.entity.BlockDisplay;
import org.bukkit.entity.EntityType;
import org.bukkit.persistence.PersistentDataContainer;
import org.bukkit.persistence.PersistentDataType;
import org.bukkit.util.Transformation;
import org.bukkit.util.Vector;
import org.jetbrains.annotations.NotNull;

import java.util.Objects;
import java.util.UUID;

public class Pixel {
    private static final NamespacedKey parentBlockTag = new NamespacedKey(Main.plugin, "parent");

    private final @NotNull PixelBlock parentBlock;
    private final @NotNull BlockDisplay entity;

    public final Location location;
    public final BlockData blockData;
    private final double scale;


    public static Pixel existingPixel(PixelBlock parentBlock, BlockDisplay entity) {
        PersistentDataContainer dataContainer = entity.getPersistentDataContainer();
        if(!dataContainer.has(parentBlockTag)) throw new IllegalArgumentException("Entity is missing the parent tag in the DataContainer");
        UUID expectedParentUuid = UUID.fromString(Objects.requireNonNull(dataContainer.get(parentBlockTag, PersistentDataType.STRING)));
        if(!parentBlock.getBlockUUID().equals(expectedParentUuid)) throw new IllegalArgumentException("Pixel is from a different parent block");
        return new Pixel(parentBlock, entity);
    }

    public static Pixel newPixel(PixelBlock parentBlock, Vector relativePosition, BlockData blockData, double scale) {
        return new Pixel(parentBlock, relativePosition, blockData, scale);
    }

    private Pixel(@NotNull PixelBlock parentBlock, @NotNull BlockDisplay entity) {
        this.parentBlock = parentBlock;
        this.entity = entity;
        this.location = entity.getLocation();
        this.blockData = entity.getBlock();
        this.scale = entity.getTransformation().getScale().get(0);
    }

    private Pixel(@NotNull PixelBlock parentBlock, @NotNull Vector relativePosition, @NotNull BlockData blockData, double scale) {
        this.parentBlock = parentBlock;
        this.location = parentBlock.getPixelBlockLocation().add(relativePosition.multiply(scale));
        this.blockData = blockData;
        this.scale = scale;
        this.entity = (BlockDisplay) this.location.getWorld().spawnEntity(this.location, EntityType.BLOCK_DISPLAY);

        this.entity.setBlock(blockData);
        Transformation transform = this.entity.getTransformation();
        transform.getScale().set(this.scale);
        this.entity.setTransformation(transform);
        this.entity.getPersistentDataContainer().set(parentBlockTag, PersistentDataType.STRING, this.parentBlock.getBlockUUID().toString());
    }

    public void destroy() {
        this.entity.remove();
    }
}
